index_code_list_stock = ["11001", "11002", "11003", "11004", "11005", "11006", "11007"]
index_code_list_bond = ["11008", "11009"]
index_code_list_derivation = ["11010", "11011", "11012", "11013", "11014"]

index_code_list = [str(code) for code in range(11001, 11014)]

stock_code_list = [str(code) for code in range(12001, 12029)]

product_code_list = [str(code) for code in range(13101, 13118)] + [str(code) for code in range(13201, 13217)] + [str(code) for code in range(13301, 13313)]


bond_code_list_price = ["14001", "14002"]
bond_code_list_info = ["14003", "14004"]
bond_code_list_trade = ["14005", "14006", "14007", "14008"]
bond_code_list_detail = [str(code) for code in range(14009, 14028)]

derivative_code_list_price = ["15001", "15002", "15003"]
derivative_code_list_info = ["15004", "15005"]
derivative_code_list_trade = ["15006", "15007", "15008", "15009"]
derivative_code_list_detail = [str(code) for code in range(15010, 15017)]

commodity_code_list_oil = [str(code) for code in range(16101, 16106)]
commodity_code_list_gold = [str(code) for code in range(16201, 16208)]
commodity_code_list_carbonemission = [str(code) for code in range(16301, 16305)]

oversees_code_list_euro = [str(code) for code in range(17101, 17109)]
